@@ -5,48 +5,33 @@ It does this via the Public Suffix List (PSL).
     >>> import tldextract
 
     >>> tldextract.extract('http://forums.news.cnn.com/')
-    ExtractResult(subdomain='forums.news', domain='cnn', suffix='com')
+    ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
 
     >>> tldextract.extract('http://forums.bbc.co.uk/') # United Kingdom
-    ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
+    ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
 
     >>> tldextract.extract('http://www.worldbank.org.kg/') # Kyrgyzstan
-    ExtractResult(subdomain='www', domain='worldbank', suffix='org.kg')
-
-`ExtractResult` is a namedtuple, so it's simple to access the parts you want.
-
-    >>> ext = tldextract.extract('http://forums.bbc.co.uk')
-    >>> (ext.subdomain, ext.domain, ext.suffix)
-    ('forums', 'bbc', 'co.uk')
-    >>> # rejoin subdomain and domain
-    >>> '.'.join(ext[:2])
-    'forums.bbc'
-    >>> # a common alias
-    >>> ext.registered_domain
-    'bbc.co.uk'
+    ExtractResult(subdomain='www', domain='worldbank', suffix='org.kg', is_private=False)
 
 Note subdomain and suffix are _optional_. Not all URL-like inputs have a
 subdomain or a valid suffix.
 
     >>> tldextract.extract('google.com')
-    ExtractResult(subdomain='', domain='google', suffix='com')
+    ExtractResult(subdomain='', domain='google', suffix='com', is_private=False)
 
     >>> tldextract.extract('google.notavalidsuffix')
-    ExtractResult(subdomain='google', domain='notavalidsuffix', suffix='')
+    ExtractResult(subdomain='google', domain='notavalidsuffix', suffix='', is_private=False)
 
     >>> tldextract.extract('http://127.0.0.1:8080/deployed/')
-    ExtractResult(subdomain='', domain='127.0.0.1', suffix='')
+    ExtractResult(subdomain='', domain='127.0.0.1', suffix='', is_private=False)
 
-If you want to rejoin the whole namedtuple, regardless of whether a subdomain
-or suffix were found:
+To rejoin the original hostname, if it was indeed a valid, registered hostname:
 
-    >>> ext = tldextract.extract('http://127.0.0.1:8080/deployed/')
-    >>> # this has unwanted dots
-    >>> '.'.join(ext)
-    '.127.0.0.1.'
-    >>> # join part only if truthy
-    >>> '.'.join(part for part in ext if part)
-    '127.0.0.1'
+    >>> ext = tldextract.extract('http://forums.bbc.co.uk')
+    >>> ext.registered_domain
+    'bbc.co.uk'
+    >>> ext.fqdn
+    'forums.bbc.co.uk'
 """
 
 from __future__ import annotations
@@ -54,22 +39,14 @@ from __future__ import annotations
 import logging
 import os
 import urllib.parse
+from collections.abc import Collection, Sequence
+from dataclasses import dataclass
 from functools import wraps
-from typing import (
-    Collection,
-    Dict,
-    FrozenSet,
-    List,
-    NamedTuple,
-    Optional,
-    Sequence,
-    Union,
-)
 
 import idna
 
 from .cache import DiskCache, get_cache_dir
-from .remote import IP_RE, lenient_netloc, looks_like_ip
+from .remote import lenient_netloc, looks_like_ip, looks_like_ipv6
 from .suffix_list import get_suffix_lists
 
 LOG = logging.getLogger("tldextract")
@@ -83,12 +60,17 @@ PUBLIC_SUFFIX_LIST_URLS = (
 )
 
 
-class ExtractResult(NamedTuple):
-    """namedtuple of a URL's subdomain, domain, and suffix."""
+@dataclass(order=True)
+class ExtractResult:
+    """A URL's extracted subdomain, domain, and suffix.
+
+    Also contains metadata, like a flag that indicates if the URL has a private suffix.
+    """
 
     subdomain: str
     domain: str
     suffix: str
+    is_private: bool
 
     @property
     def registered_domain(self) -> str:
@@ -114,10 +96,8 @@ class ExtractResult(NamedTuple):
         >>> extract('http://localhost:8080').fqdn
         ''
         """
-        if self.suffix and self.domain:
-            # Disable bogus lint error (https://github.com/PyCQA/pylint/issues/2568)
-            # pylint: disable-next=not-an-iterable
-            return ".".join(i for i in self if i)
+        if self.suffix and (self.domain or self.is_private):
+            return ".".join(i for i in (self.subdomain, self.domain, self.suffix) if i)
         return ""
 
     @property
@@ -132,23 +112,51 @@ class ExtractResult(NamedTuple):
         >>> extract('http://256.1.1.1').ipv4
         ''
         """
-        if not (self.suffix or self.subdomain) and IP_RE.match(self.domain):
+        if (
+            self.domain
+            and not (self.suffix or self.subdomain)
+            and looks_like_ip(self.domain)
+        ):
             return self.domain
         return ""
 
+    @property
+    def ipv6(self) -> str:
+        """
+        Returns the ipv6 if that is what the presented domain/url is.
+
+        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1]/path/to/file').ipv6
+        'aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1'
+        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:127.0.0.1.1]/path/to/file').ipv6
+        ''
+        >>> extract('http://[aBcD:ef01:2345:6789:aBcD:ef01:256.0.0.1]').ipv6
+        ''
+        """
+        min_num_ipv6_chars = 4
+        if (
+            len(self.domain) >= min_num_ipv6_chars
+            and self.domain[0] == "["
+            and self.domain[-1] == "]"
+            and not (self.suffix or self.subdomain)
+        ):
+            debracketed = self.domain[1:-1]
+            if looks_like_ipv6(debracketed):
+                return debracketed
+        return ""
+
 
 class TLDExtract:
     """A callable for extracting, subdomain, domain, and suffix components from a URL."""
 
-    # TODO: Agreed with Pylint: too-many-arguments
-    def __init__(  # pylint: disable=too-many-arguments
+    # TODO: too-many-arguments
+    def __init__(
         self,
-        cache_dir: Optional[str] = get_cache_dir(),
+        cache_dir: str | None = get_cache_dir(),
         suffix_list_urls: Sequence[str] = PUBLIC_SUFFIX_LIST_URLS,
         fallback_to_snapshot: bool = True,
         include_psl_private_domains: bool = False,
         extra_suffixes: Sequence[str] = (),
-        cache_fetch_timeout: Union[str, float, None] = CACHE_TIMEOUT,
+        cache_fetch_timeout: str | float | None = CACHE_TIMEOUT,
     ) -> None:
         """Construct a callable for extracting subdomain, domain, and suffix components from a URL.
 
@@ -203,7 +211,7 @@ class TLDExtract:
 
         self.include_psl_private_domains = include_psl_private_domains
         self.extra_suffixes = extra_suffixes
-        self._extractor: Optional[_PublicSuffixListTLDExtractor] = None
+        self._extractor: _PublicSuffixListTLDExtractor | None = None
 
         self.cache_fetch_timeout = (
             float(cache_fetch_timeout)
@@ -227,16 +235,16 @@ class TLDExtract:
 
         >>> extractor = TLDExtract()
         >>> extractor.extract_str('http://forums.news.cnn.com/')
-        ExtractResult(subdomain='forums.news', domain='cnn', suffix='com')
+        ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
         >>> extractor.extract_str('http://forums.bbc.co.uk/')
-        ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
+        ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
         """
         return self._extract_netloc(lenient_netloc(url), include_psl_private_domains)
 
     def extract_urllib(
         self,
-        url: Union[urllib.parse.ParseResult, urllib.parse.SplitResult],
-        include_psl_private_domains: Optional[bool] = None,
+        url: urllib.parse.ParseResult | urllib.parse.SplitResult,
+        include_psl_private_domains: bool | None = None,
     ) -> ExtractResult:
         """Take the output of urllib.parse URL parsing methods and further splits the parsed URL.
 
@@ -248,33 +256,46 @@ class TLDExtract:
 
         >>> extractor = TLDExtract()
         >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.news.cnn.com/'))
-        ExtractResult(subdomain='forums.news', domain='cnn', suffix='com')
+        ExtractResult(subdomain='forums.news', domain='cnn', suffix='com', is_private=False)
         >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.bbc.co.uk/'))
-        ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
+        ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk', is_private=False)
         """
         return self._extract_netloc(url.netloc, include_psl_private_domains)
 
     def _extract_netloc(
-        self, netloc: str, include_psl_private_domains: Optional[bool]
+        self, netloc: str, include_psl_private_domains: bool | None
     ) -> ExtractResult:
-        labels = (
+        netloc_with_ascii_dots = (
             netloc.replace("\u3002", "\u002e")
             .replace("\uff0e", "\u002e")
             .replace("\uff61", "\u002e")
-            .split(".")
         )
 
-        suffix_index = self._get_tld_extractor().suffix_index(
+        min_num_ipv6_chars = 4
+        if (
+            len(netloc_with_ascii_dots) >= min_num_ipv6_chars
+            and netloc_with_ascii_dots[0] == "["
+            and netloc_with_ascii_dots[-1] == "]"
+        ):
+            if looks_like_ipv6(netloc_with_ascii_dots[1:-1]):
+                return ExtractResult("", netloc_with_ascii_dots, "", is_private=False)
+
+        labels = netloc_with_ascii_dots.split(".")
+
+        suffix_index, is_private = self._get_tld_extractor().suffix_index(
             labels, include_psl_private_domains=include_psl_private_domains
         )
 
-        if suffix_index == len(labels) and netloc and looks_like_ip(netloc):
-            return ExtractResult("", netloc, "")
+        num_ipv4_labels = 4
+        if suffix_index == len(labels) == num_ipv4_labels and looks_like_ip(
+            netloc_with_ascii_dots
+        ):
+            return ExtractResult("", netloc_with_ascii_dots, "", is_private)
 
         suffix = ".".join(labels[suffix_index:]) if suffix_index != len(labels) else ""
         subdomain = ".".join(labels[: suffix_index - 1]) if suffix_index >= 2 else ""
         domain = labels[suffix_index - 1] if suffix_index else ""
-        return ExtractResult(subdomain, domain, suffix)
+        return ExtractResult(subdomain, domain, suffix, is_private)
 
     def update(self, fetch_now: bool = False) -> None:
         """Force fetch the latest suffix list definitions."""
@@ -284,7 +305,7 @@ class TLDExtract:
             self._get_tld_extractor()
 
     @property
-    def tlds(self) -> List[str]:
+    def tlds(self) -> list[str]:
         """
         Returns the list of tld's used by default.
 
@@ -331,44 +352,58 @@ TLD_EXTRACTOR = TLDExtract()
 class Trie:
     """Trie for storing eTLDs with their labels in reverse-order."""
 
-    def __init__(self, matches: Optional[Dict] = None, end: bool = False) -> None:
+    def __init__(
+        self, matches: dict | None = None, end: bool = False, is_private: bool = False
+    ) -> None:
+        """TODO."""
         self.matches = matches if matches else {}
         self.end = end
+        self.is_private = is_private
 
     @staticmethod
-    def create(suffixes: Collection[str]) -> Trie:
+    def create(
+        public_suffixes: Collection[str],
+        private_suffixes: Collection[str] | None = None,
+    ) -> Trie:
         """Create a Trie from a list of suffixes and return its root node."""
         root_node = Trie()
 
-        for suffix in suffixes:
-            suffix_labels = suffix.split(".")
-            suffix_labels.reverse()
-            root_node.add_suffix(suffix_labels)
+        for suffix in public_suffixes:
+            root_node.add_suffix(suffix)
+
+        if private_suffixes is None:
+            private_suffixes = []
+
+        for suffix in private_suffixes:
+            root_node.add_suffix(suffix, True)
 
         return root_node
 
-    def add_suffix(self, labels: List[str]) -> None:
+    def add_suffix(self, suffix: str, is_private: bool = False) -> None:
         """Append a suffix's labels to this Trie node."""
         node = self
 
+        labels = suffix.split(".")
+        labels.reverse()
+
         for label in labels:
             if label not in node.matches:
                 node.matches[label] = Trie()
             node = node.matches[label]
 
         node.end = True
+        node.is_private = is_private
 
 
 @wraps(TLD_EXTRACTOR.__call__)
-def extract(  # pylint: disable=missing-function-docstring
-    url: str, include_psl_private_domains: Optional[bool] = False
+def extract(  # noqa: D103
+    url: str, include_psl_private_domains: bool | None = False
 ) -> ExtractResult:
     return TLD_EXTRACTOR(url, include_psl_private_domains=include_psl_private_domains)
 
 
 @wraps(TLD_EXTRACTOR.update)
-# pylint: disable-next=missing-function-docstring
-def update(*args, **kwargs):  # type: ignore[no-untyped-def]
+def update(*args, **kwargs):  # type: ignore[no-untyped-def]  # noqa: D103
     return TLD_EXTRACTOR.update(*args, **kwargs)
 
 
@@ -377,9 +412,9 @@ class _PublicSuffixListTLDExtractor:
 
     def __init__(
         self,
-        public_tlds: List[str],
-        private_tlds: List[str],
-        extra_tlds: List[str],
+        public_tlds: list[str],
+        private_tlds: list[str],
+        extra_tlds: list[str],
         include_psl_private_domains: bool = False,
     ):
         # set the default value
@@ -388,12 +423,12 @@ class _PublicSuffixListTLDExtractor:
         self.private_tlds = private_tlds
         self.tlds_incl_private = frozenset(public_tlds + private_tlds + extra_tlds)
         self.tlds_excl_private = frozenset(public_tlds + extra_tlds)
-        self.tlds_incl_private_trie = Trie.create(self.tlds_incl_private)
+        self.tlds_incl_private_trie = Trie.create(
+            self.tlds_excl_private, frozenset(private_tlds)
+        )
         self.tlds_excl_private_trie = Trie.create(self.tlds_excl_private)
 
-    def tlds(
-        self, include_psl_private_domains: Optional[bool] = None
-    ) -> FrozenSet[str]:
+    def tlds(self, include_psl_private_domains: bool | None = None) -> frozenset[str]:
         """Get the currently filtered list of suffixes."""
         if include_psl_private_domains is None:
             include_psl_private_domains = self.include_psl_private_domains
@@ -405,9 +440,9 @@ class _PublicSuffixListTLDExtractor:
         )
 
     def suffix_index(
-        self, spl: List[str], include_psl_private_domains: Optional[bool] = None
-    ) -> int:
-        """Return the index of the first suffix label.
+        self, spl: list[str], include_psl_private_domains: bool | None = None
+    ) -> tuple[int, bool]:
+        """Return the index of the first suffix label, and whether it is private.
 
         Returns len(spl) if no suffix is found.
         """
@@ -425,21 +460,21 @@ class _PublicSuffixListTLDExtractor:
             decoded_label = _decode_punycode(label)
             if decoded_label in node.matches:
                 j -= 1
-                if node.matches[decoded_label].end:
-                    i = j
                 node = node.matches[decoded_label]
+                if node.end:
+                    i = j
                 continue
 
             is_wildcard = "*" in node.matches
             if is_wildcard:
                 is_wildcard_exception = "!" + decoded_label in node.matches
                 if is_wildcard_exception:
-                    return j
-                return j - 1
+                    return j, node.matches["*"].is_private
+                return j - 1, node.matches["*"].is_private
 
             break
 
-        return i
+        return i, node.is_private
 
 
 def _decode_punycode(label: str) -> str:
